#version 330

uniform sampler2D colorTexture;
uniform sampler2DShadow shadowMap;
uniform vec3 lightPosition;
uniform vec3 ambientPercentage;
uniform vec3 specularPercentage;
uniform int enableDiscard;

out vec4 fragColor;

in vec3 worldNormal;
in vec2 fUVs;
in vec4 shadowCoord;


in vec3 worldPosition;
in vec3 normalCamera;
in vec3 eyeDirection;
in vec3 lightDirection;

vec2 poissonDisk[16] = vec2[]( 
   vec2( -0.94201624, -0.39906216 ), 
   vec2( 0.94558609, -0.76890725 ), 
   vec2( -0.094184101, -0.92938870 ), 
   vec2( 0.34495938, 0.29387760 ), 
   vec2( -0.91588581, 0.45771432 ), 
   vec2( -0.81544232, -0.87912464 ), 
   vec2( -0.38277543, 0.27676845 ), 
   vec2( 0.97484398, 0.75648379 ), 
   vec2( 0.44323325, -0.97511554 ), 
   vec2( 0.53742981, -0.47373420 ), 
   vec2( -0.26496911, -0.41893023 ), 
   vec2( 0.79197514, 0.19090188 ), 
   vec2( -0.24188840, 0.99706507 ), 
   vec2( -0.81409955, 0.91437590 ), 
   vec2( 0.19984126, 0.78641367 ), 
   vec2( 0.14383161, -0.14100790 ) 
);


void main()
{

	vec3 lColor = vec3(1,1,1);
	float lIntensity = 10.0;

	//TODO ambient und specular color als uniform
	vec3 diffuseColor = texture(colorTexture, fUVs).rgb;
	vec3 ambientColor = ambientPercentage * diffuseColor;
	vec3 specularColor = specularPercentage;

	// Distanz zum Licht
	float distance = length(lightPosition - worldPosition);

	// Normale
	vec3 n = normalize(normalCamera);

	// Lichtrichtung
	vec3 l = normalize(lightDirection);

	float cosTheta = clamp(dot(n,l), 0,1);
	
	// Auge
	vec3 E = normalize(eyeDirection);

	// Richtung Reflektiertes Licht
	vec3 R = reflect(-l,n);
	float cosAlpha = clamp(dot(E,R), 0,1);

	float visibility=1;

	float bias = 0.005;


	for (int i=0;i<4;i++){
		int index = i;
		visibility -= 0.2*(1.0 - texture( shadowMap, vec3(shadowCoord.xy + poissonDisk[index]/700.0,  (shadowCoord.z-bias)/shadowCoord.w) ));
	}
	
	// Farbe die angezeigt wird mit diffus, ambient und specular
	//vec3 color = ambientColor +	diffuseColor * lColor * lIntensity * cosTheta / (distance*distance) +
	//	specularColor * lColor * lIntensity * pow(cosAlpha,5) / (distance*distance);

	vec3 color = ambientColor +	visibility * diffuseColor * lColor * lIntensity * cosTheta / (distance*distance) +
				 visibility * specularColor * lColor * lIntensity * pow(cosAlpha,5) / (distance*distance);

	fragColor = vec4(color,1.0);

	if(worldPosition.y < -0.61 && enableDiscard == 1){
			discard;
	}
}