#version 330

uniform sampler2D colorTexture;
uniform sampler2D normalTexture;
uniform vec3 lightPosition;
uniform vec3 ambientPercentage;
uniform vec3 specularPercentage;
uniform int enableDiscard;

out vec4 fragColor;

in vec3 worldNormal;
in vec2 fUVs;


in vec3 worldPosition;
in vec3 eyeDirectionTangent;
in vec3 lightDirectionTangent;

void main()
{
	
	//Nur textur ohne licht
	//vec3 color = texture(colorTexture, fUVs).rgb;
	//fragColor = vec4(color, 1.0);

	//TODO lichtfarbe und intensitt als uniform

	vec3 lColor = vec3(1,1,1);
	float lIntensity = 40.0;

	//TODO ambient und specular color als uniform
	vec3 diffuseColor = texture(colorTexture, fUVs*3).rgb;
	vec3 ambientColor = ambientPercentage * diffuseColor;
	//vec3 specularColor = specularPercentage;
	vec3 specularColor = vec3(0,0,0);
	//Normalmap Textur
	vec3 normalTextureTangent = normalize(texture2D(normalTexture, fUVs*3 ).rgb*2.0 - 1.0);

	// Distanz zum Licht
	float distance = length(lightPosition - worldPosition);

	// Normale
	vec3 n = normalTextureTangent;

	// Lichtrichtung
	vec3 l = normalize(lightDirectionTangent);

	float cosTheta = clamp(dot(n,l), 0,1);
	
	// Auge
	vec3 E = normalize(eyeDirectionTangent);

	// Richtung Reflektiertes Licht
	vec3 R = reflect(-l,n);
	float cosAlpha = clamp(dot(E,R), 0,1);
	
	// Farbe die angezeigt wird mit diffus, ambient und specular
	vec3 color = ambientColor +	diffuseColor * lColor * lIntensity * cosTheta / (distance*distance) +
		specularColor * lColor * lIntensity * pow(cosAlpha,5) / (distance*distance);

	fragColor = vec4(color,1.0);
	//fragColor = vec4(lightDirectionTangent,1.0);

	if(worldPosition.y < -0.61 && enableDiscard == 1){
			discard;
	}
}