#version 330

uniform sampler2D colorTexture;
uniform sampler2D normalTexture;
uniform sampler2D reflectionTexture;
uniform sampler2D dudv;
uniform sampler2DShadow shadowMap;
uniform vec3 lightPosition;
uniform vec3 ambientPercentage;
uniform vec3 specularPercentage;
uniform float time;
uniform int transparencyMode;

out vec4 fragColor;

in vec3 eyeDirection;
in vec3 normalCameraa;
in vec3 lightDirection;

in vec3 worldNormal;
in vec2 fUVs;
in vec4 shadowCoord;
in vec4 reflectCoord;

in vec3 worldPosition;
in vec3 eyeDirectionTangent;
in vec3 lightDirectionTangent;

vec2 poissonDisk[16] = vec2[]( 
   vec2( -0.94201624, -0.39906216 ), 
   vec2( 0.94558609, -0.76890725 ), 
   vec2( -0.094184101, -0.92938870 ), 
   vec2( 0.34495938, 0.29387760 ), 
   vec2( -0.91588581, 0.45771432 ), 
   vec2( -0.81544232, -0.87912464 ), 
   vec2( -0.38277543, 0.27676845 ), 
   vec2( 0.97484398, 0.75648379 ), 
   vec2( 0.44323325, -0.97511554 ), 
   vec2( 0.53742981, -0.47373420 ), 
   vec2( -0.26496911, -0.41893023 ), 
   vec2( 0.79197514, 0.19090188 ), 
   vec2( -0.24188840, 0.99706507 ), 
   vec2( -0.81409955, 0.91437590 ), 
   vec2( 0.19984126, 0.78641367 ), 
   vec2( 0.14383161, -0.14100790 ) 
);

float distortion = 0.085;

void main()
{
	
	//Nur textur ohne licht
	//vec3 color = texture(colorTexture, fUVs).rgb;
	//fragColor = vec4(color, 1.0);

	//TODO lichtfarbe und intensitt als uniform

	vec3 lColor = vec3(0.99,0.98,0.68);
	float lIntensity = 30.0;


	

	//Normalmap Textur
	vec2 normalfUVs = fUVs;
	normalfUVs.s -= time;
	normalfUVs.t -= time;

	vec4 offset = texture(dudv, normalfUVs * 7 + vec2(time)) * distortion;
	vec4 dudvColor = texture(dudv, vec2(normalfUVs * 7 + offset.xy));
    dudvColor = normalize(dudvColor * 2.0 - 1.0) * 0.009;

	vec2 screenPos = reflectCoord.xy / reflectCoord.w;
	screenPos = screenPos / 2 + vec2(0.5,0.5);
	screenPos += dudvColor.xy;
	screenPos = clamp(screenPos, 0.001, 0.999);

	//reflection
    vec3 reflection = texture(reflectionTexture, screenPos.xy).rgb;

	//TODO ambient und specular color als uniform
	//vec3 diffuseColor = texture(colorTexture, fUVs * 7).rgb;
	vec3 diffuseColor = vec3(0.51,0.85,0.97);
	diffuseColor = diffuseColor * 0.6 + reflection * 2.0;

	vec3 ambientColor = vec3(0.4,0.4,0.4) * diffuseColor;
	//vec3 specularColor = specularPercentage;
	vec3 specularColor = vec3(0.3,0.3,0.3);
	//vec3 ambientColor = vec3(0.7,0.7,0.7) * diffuseColor;
	//vec3 specularColor = vec3(0.3,0.3,0.3);



	vec3 normalTextureTangent = normalize(texture2D(normalTexture, normalfUVs * 7 + offset.xy ).rgb*2.0 - 1.0);

	// Distanz zum Licht
	float distance = length(lightPosition - worldPosition);

	// Normale
	vec3 n = normalize(normalTextureTangent);

	// Lichtrichtung
	vec3 l = normalize(lightDirectionTangent);

	float cosTheta = clamp(dot(n,l), 0,1);


	// Normale
	vec3 nn = normalize(normalCameraa);

	// Lichtrichtung
	vec3 ll = normalize(lightDirection);

	float cosThetaa = clamp(dot(nn,ll), 0,1);
	
	// Auge
	vec3 E = normalize(eyeDirectionTangent);

	// Richtung Reflektiertes Licht
	vec3 R = reflect(-l,n);
	float cosAlpha = clamp(dot(E,R), 0,1);

	//correction
	float fresnel = clamp(dot(normalize(normalCameraa),normalize(eyeDirection)), 0,1);
	float fresnelTerm =1-fresnel;

	float visibility=1;
	float bias = 0.005;

	for (int i=0;i<8;i++){
		int index = i;
		visibility -= 0.3*(1.0 - texture( shadowMap, vec3(shadowCoord.xy + poissonDisk[index]/900.0,  (shadowCoord.z-bias)/shadowCoord.w) ));
	}

	
	
	// Farbe die angezeigt wird mit diffus, ambient und specular
	//vec3 color = ambientColor +	diffuseColor * lColor * lIntensity * cosTheta / (distance*distance) +
	//	specularColor * lColor * lIntensity * pow(cosAlpha,5) / (distance*distance);

	vec3 color = ambientColor +	visibility * diffuseColor * lColor * lIntensity * cosTheta / (distance*distance) +
				 visibility * specularColor * lColor * lIntensity * pow(cosAlpha,5) / (distance*distance);

	
	// Farbe die angezeigt wird mit diffus, ambient und specular
	//vec3 color = ambientColor +	diffuseColor * lColor * lIntensity * cosTheta / (distance*distance) +
		//specularColor * lColor * lIntensity * pow(cosAlpha,5) / (distance*distance);

	//fragColor = vec4(0.0,0.4,1.0,0.5);
   //vec3 colors =  texture2D(colorTexture, fUVs).rgb;
	//fragColor = vec4(colors,0.6);

	
	//fragColor = vec4(color,fresnelTerm) +  vec4(0.51,0.85,0.97,(1-fresnelTerm));
	if(transparencyMode == 1){
		fragColor = vec4(color,fresnelTerm);
	}else if(transparencyMode == 0){
		fragColor = vec4(color,1.0);
	}
	

	
}